from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject

_EVG_DATE_FIELDS_IN_PATCH = frozenset([
    'create_time',
    'start_time',
    'finish_time',
])


class GithubPatchData(_BaseEvergreenObject):
    """Representation of github patch data in a patch object."""

    def __init__(self, json, api):
        """
        Create an instance of github patch data.

        :param json: json representing github patch data.
        :param api: instance of evergreen api object.
        """
        super(GithubPatchData, self).__init__(json, api)


class VariantsTasks(_BaseEvergreenObject):
    """Representation of a variants tasks object."""

    def __init__(self, json, api):
        """
        Create an instance of a variants tasks object.

        :param json: json representing variants tasks object.
        :param api: intance of the evergreen api object.
        """
        super(VariantsTasks, self).__init__(json, api)
        self._task_set = 0

    @property
    def tasks(self):
        """
        Retrieve the set of all tasks for this variant.

        :return: Set of all tasks for this variant.
        """
        if not self._task_set:
            self._task_set = set(self.json['tasks'])

        return self._task_set


class Patch(_BaseEvergreenObject):
    """
    Representation of an Evergreen patch.
    """

    def __init__(self, json, api):
        """
        Create an instance of an evergreen patch.

        :param json: json representing patch.
        """
        super(Patch, self).__init__(json, api)
        self._date_fields = _EVG_DATE_FIELDS_IN_PATCH
        self._variants_tasks = None
        self._variant_task_dict = None

    @property
    def github_patch_data(self):
        """
        Retrieve the github patch data for this patch.

        :return: Github Patch Data for this patch.
        """
        return GithubPatchData(self.json['github_patch_data'], self._api)

    @property
    def variants_tasks(self):
        """
        Retrieve the variants tasks for this patch.

        :return: variants tasks for this patch.
        """
        if not self._variants_tasks:
            self._variants_tasks = [VariantsTasks(vt, self._api)
                                    for vt in self.json['variants_tasks']]
        return self._variants_tasks

    def task_list_for_variant(self, variant):
        """
        Retrieve the list of tasks for the given variant.

        :param variant: name of variant to search for.
        :return: list of tasks belonging to the specified variant.
        """
        if not self._variant_task_dict:
            self._variant_task_dict = {vt.name: vt.tasks for vt in self.variants_tasks}
        return self._variant_task_dict[variant]

    def get_version(self):
        """
        Get version for this patch.

        :return: Version object.
        """
        return self._api.version_by_id(self.version)

    def __str__(self):
        return '{}: {}'.format(self.patch_id, self.description)



from __future__ import absolute_import

import click

from evergreen.api import EvergreenApi

project_id = 'mongodb-mongo-master'


@click.group()
@click.pass_context
def cli(ctx):
    ctx.ensure_object(dict)
    ctx.obj['api'] = EvergreenApi.get_api(use_config_file=True)


@cli.command()
@click.pass_context
def list_hosts(ctx):
    api = ctx.obj['api']
    host_list = api.get_all_hosts()
    for host in host_list:
        click.echo(host)


@cli.command()
@click.pass_context
@click.option('-p', '--project', required=True)
@click.option('-l', '--limit', type=int)
def list_patches(ctx, project, limit):
    api = ctx.obj['api']
    for i, p in enumerate(api.patches_by_project(project)):
        click.echo(p)
        if limit and i > limit:
            break


@cli.command()
@click.pass_context
def list_projects(ctx):
    api = ctx.obj['api']
    project_list = api.get_all_projects()
    click.echo('Project ID:\tProject Name')
    click.echo('-----------\t------------')
    for project in project_list:
        click.echo('{}: {}'.format(project.identifier, project.display_name))


@cli.command()
@click.pass_context
@click.option('-a', '--after-date', required=True)
@click.option('-b', '--before-date', required=True)
@click.option('-p', '--project', required=True)
@click.option('-d', '--distros', multiple=True)
@click.option('--group-by')
@click.option('-g', '--group-num-days')
@click.option('-r', '--requesters', multiple=True)
@click.option('-s', '--sort')
@click.option('--tests', multiple=True)
@click.option('-t', '--tasks', multiple=True)
@click.option('-v', '--variants', multiple=True)
def test_stats(ctx, after_date, before_date, project, distros, group_by, group_num_days, requesters,
               sort, tests, tasks, variants):
    api = ctx.obj['api']

    test_stat_list = api.test_stats_by_project(project, after_date, before_date, group_num_days,
                                               requesters, tests, tasks, variants, distros,
                                               group_by, sort)
    for t in test_stat_list:
        print('{} - {}:'.format(t.task_name, t.test_file))
        print('\tpass: {}, fail: {}'.format(t.num_pass, t.num_fail))
        print('\taverage runtime: {}'.format(t.avg_duration_pass))


def main():
    return cli(obj={})



# -*- encoding: utf-8 -*-
"""Task representation of evergreen."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject

EVG_SUCCESS_STATUS = 'success'
EVG_SYSTEM_FAILURE_STATUS = 'system'

_EVG_DATE_FIELDS_IN_TASK = frozenset([
    'create_time',
    'dispatch_time',
    'finish_time',
    'ingest_time',
    'scheduled_time',
    'start_time',
])


class Task(_BaseEvergreenObject):
    """Representation of an Evergreen task."""

    def __init__(self, json, api):
        """
        Create an instance of an evergreen task.
        """
        super(Task, self).__init__(json, api)
        self._date_fields = _EVG_DATE_FIELDS_IN_TASK

    def get_execution(self, execution):
        """
        Get the task info for the specified execution.

        :param execution: Index of execution.
        :return: Task info for specified execution.
        """
        if self.execution == execution:
            return self

        if 'previous_executions' in self.json:
            for task in self.json['previous_executions']:
                if task['execution'] == execution:
                    return Task(task, self._api)

        return None

    def wait_time(self):
        """
        Get the time taken until the task started running.

        :return: Time taken until task started running.
        """
        if self.start_time and self.ingest_time:
            return self.start_time - self.ingest_time
        return None

    def is_success(self):
        """
        Whether task was successful.

        :return: True if task was successful.
        """
        return self.status == EVG_SUCCESS_STATUS

    def is_system_failure(self):
        """
        Whether task resulted in a system failure.

        :return: True if task was a system failure.
        """
        if not self.is_success() and self.status_details and self.status_details['type']:
            return self.status_details['type'] == EVG_SYSTEM_FAILURE_STATUS
        return False

    def is_timeout(self):
        """
        Whether task results in a timeout.

        :return: True if task was a timeout.
        """
        if not self.is_success() and self.status_details and self.status_details['timed_out']:
            return self.status_details['timed_out']
        return False



# -*- encoding: utf-8 -*-
"""Task representation of evergreen."""
from __future__ import absolute_import

from evergreen.util import parse_evergreen_datetime


class _BaseEvergreenObject(object):
    """Common evergreen object."""

    def __init__(self, json, api):
        """
        Create an instance of an evergreen task.
        """
        self.json = json
        self._api = api
        self._date_fields = None

    def _is_field_a_date(self, item):
        return self._date_fields and item in self._date_fields and self.json[item]

    def __getattr__(self, item):
        """Lookup an attribute if it exists."""
        if item in self.json:
            if self._is_field_a_date(item):
                return parse_evergreen_datetime(self.json[item])
            return self.json[item]
        raise TypeError('Unknown attribute {0}'.format(item))



# -*- encoding: utf-8 -*-
"""Version representation of evergreen."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject

_EVG_DATE_FIELDS_IN_VERSION = frozenset([
    'create_time',
    'finish_time',
    'start_time',
])


class BuildVariantStatus(_BaseEvergreenObject):
    def __init__(self, json, api):
        super(BuildVariantStatus, self).__init__(json, api)

    def get_build(self):
        return self._api.build_by_id(self.build_id)


class Version(_BaseEvergreenObject):
    def __init__(self, json, api):
        """
        Create an instance of an evergreen version.

        :param json: json representing version
        """
        super(Version, self).__init__(json, api)
        self._date_fields = _EVG_DATE_FIELDS_IN_VERSION

        if 'build_variants_status' in self.json:
            self.build_variants_map = {
                bvs['build_variant']: bvs['build_id']
                for bvs in self.json['build_variants_status']
            }

    @property
    def build_variants_status(self):
        build_variants_status = self.json['build_variants_status']
        return [BuildVariantStatus(bvs, self._api) for bvs in build_variants_status]

    def build_by_variant(self, build_variant):
        """
        Get a build object for the specified variant.

        :param build_variant: Build variant to get build for.
        :return: Build object for variant.
        """
        return self._api.build_by_id(self.build_variants_map[build_variant])

    def get_manifest(self):
        """
        Get the manifest for this version.

        :return: Manifest for this version.
        """
        return self._api.manifest(self.project, self.revision)
